import java.awt.MouseInfo;
import java.awt.event.KeyEvent;
import java.util.HashMap;


public class Engine implements Runnable {
	
	private EngineCommand commands[];
	private int threadIndex;
	private int previousIndex[];
	private int incursionCount;
	private Command mainThread[];
	private HashMap<String, Function> functions;
	private HashMap<String, String> variables;
	private int mainThreadLength;
	HashMap<String, Integer> keys;
	boolean jumpingIf;
	boolean paused;
	Thread scriptThread;
	
	public Engine() {
		commands = new EngineCommand[49];
		commands[0] = new SetMousePosition();
		commands[1] = new Wait();
		commands[2] = new StartFunction();
		commands[3] = new EndFunction();
		commands[4] = new InScriptFunction();
		commands[5] = new Variable();
		commands[6] = new IfEquals();
		commands[7] = new EndIf();
		commands[8] = new AddInt();
		commands[9] = new AddFloat();
		commands[10] = new Add();
		commands[11] = new IfNotEqual();
		commands[12] = new IfBigger();
		commands[13] = new IfSmaller();
		commands[14] = new Key();
		commands[15] = new MouseDown();
		commands[16] = new MouseUp();
		commands[17] = new TweenMouse();
		commands[18] = new PopUp();
		commands[19] = new SetScriptText();
		commands[20] = new GetMouseX();
		commands[21] = new GetMouseY();
		commands[22] = new InputDialog();
		commands[23] = new GetFromArray();
		commands[24] = new Multiply();
		commands[25] = new Divide();
		commands[26] = new Round();
		commands[27] = new Sin();
		commands[28] = new Cos();
		commands[29] = new GetPixel();
		commands[30] = new GetRed();
		commands[31] = new GetGreen();
		commands[32] = new GetBlue();
		commands[33] = new FindColor();
		commands[34] = new GetX();
		commands[35] = new GetY();
		commands[36] = new OptionDialog();
		commands[37] = new RandomInt();
		commands[38] = new RandomFloat();
		commands[39] = new GetScreenWidth();
		commands[40] = new GetScreenHeight();
		commands[41] = new Return();
		commands[42] = new SubInt();
		commands[43] = new StringToArray();
		commands[44] = new ToLowerCase();
		commands[45] = new ToUpperCase();
		commands[46] = new GetArrayLength();
		commands[47] = new WobbleMouse();
		commands[48] = new TypeString();
		
		paused = false;
		
		keys = new HashMap<String,Integer>();
		keys.put("0",KeyEvent.VK_0);
		keys.put("1",KeyEvent.VK_1);
		keys.put("2",KeyEvent.VK_2);
		keys.put("3",KeyEvent.VK_3);
		keys.put("4",KeyEvent.VK_4);
		keys.put("5",KeyEvent.VK_5);
		keys.put("6",KeyEvent.VK_6);
		keys.put("7",KeyEvent.VK_7);
		keys.put("8",KeyEvent.VK_8);
		keys.put("9",KeyEvent.VK_9);
		keys.put("a",KeyEvent.VK_A);
		keys.put("b",KeyEvent.VK_B);
		keys.put("c",KeyEvent.VK_C);
		keys.put("d",KeyEvent.VK_D);
		keys.put("e",KeyEvent.VK_E);
		keys.put("f",KeyEvent.VK_F);
		keys.put("g",KeyEvent.VK_G);
		keys.put("h",KeyEvent.VK_H);
		keys.put("i",KeyEvent.VK_I);
		keys.put("j",KeyEvent.VK_J);
		keys.put("k",KeyEvent.VK_K);
		keys.put("l",KeyEvent.VK_L);
		keys.put("m",KeyEvent.VK_M);
		keys.put("n",KeyEvent.VK_N);
		keys.put("o",KeyEvent.VK_O);
		keys.put("p",KeyEvent.VK_P);
		keys.put("q",KeyEvent.VK_Q);
		keys.put("r",KeyEvent.VK_R);
		keys.put("s",KeyEvent.VK_S);
		keys.put("t",KeyEvent.VK_T);
		keys.put("u",KeyEvent.VK_U);
		keys.put("v",KeyEvent.VK_V);
		keys.put("w",KeyEvent.VK_W);
		keys.put("x",KeyEvent.VK_X);
		keys.put("y",KeyEvent.VK_Y);
		keys.put("z",KeyEvent.VK_Z);
		keys.put(" ",KeyEvent.VK_SPACE);
		keys.put("(",KeyEvent.VK_LEFT_PARENTHESIS);
		keys.put(")",KeyEvent.VK_RIGHT_PARENTHESIS);
		keys.put(";",KeyEvent.VK_SEMICOLON);
		keys.put(",",KeyEvent.VK_COMMA);
		keys.put(".",KeyEvent.VK_PERIOD);
		keys.put("/",KeyEvent.VK_SLASH);
		keys.put("\\",KeyEvent.VK_BACK_SLASH);
		keys.put(":",KeyEvent.VK_COLON);
		keys.put(Character.toString('"'),KeyEvent.VK_QUOTEDBL);
		keys.put("`",KeyEvent.VK_BACK_QUOTE);
		keys.put("^",KeyEvent.VK_CIRCUMFLEX);
		keys.put("[",KeyEvent.VK_OPEN_BRACKET);
		keys.put("]",KeyEvent.VK_CLOSE_BRACKET);
		keys.put("-",KeyEvent.VK_MINUS);
		keys.put("=",KeyEvent.VK_EQUALS);
		keys.put("alt",KeyEvent.VK_ALT);
		keys.put("esc",KeyEvent.VK_ESCAPE);
		keys.put("tab",KeyEvent.VK_TAB);
		keys.put("del",KeyEvent.VK_DELETE);
		keys.put("ctrl",KeyEvent.VK_CONTROL);
		keys.put("windows",KeyEvent.VK_WINDOWS);
		keys.put("enter",KeyEvent.VK_ENTER);
		keys.put("shift",KeyEvent.VK_SHIFT);
		keys.put("capslock",KeyEvent.VK_CAPS_LOCK);
		keys.put("printscreen",KeyEvent.VK_PRINTSCREEN);
		keys.put("num0",KeyEvent.VK_NUMPAD0);
		keys.put("num1",KeyEvent.VK_NUMPAD1);
		keys.put("num2",KeyEvent.VK_NUMPAD2);
		keys.put("num3",KeyEvent.VK_NUMPAD3);
		keys.put("num4",KeyEvent.VK_NUMPAD4);
		keys.put("num5",KeyEvent.VK_NUMPAD5);
		keys.put("num6",KeyEvent.VK_NUMPAD6);
		keys.put("num7",KeyEvent.VK_NUMPAD7);
		keys.put("num8",KeyEvent.VK_NUMPAD8);
		keys.put("num9",KeyEvent.VK_NUMPAD9);
		keys.put("right",KeyEvent.VK_RIGHT);
		keys.put("left",KeyEvent.VK_LEFT);
		keys.put("up",KeyEvent.VK_UP);
		keys.put("down",KeyEvent.VK_DOWN);
		keys.put("f1",KeyEvent.VK_F1);
		keys.put("f2",KeyEvent.VK_F2);
		keys.put("f3",KeyEvent.VK_F3);
		keys.put("f4",KeyEvent.VK_F4);
		keys.put("f5",KeyEvent.VK_F5);
		keys.put("f6",KeyEvent.VK_F6);
		keys.put("f7",KeyEvent.VK_F7);
		keys.put("f8",KeyEvent.VK_F8);
		keys.put("f9",KeyEvent.VK_F9);
		keys.put("f10",KeyEvent.VK_F10);
		keys.put("f11",KeyEvent.VK_F11);
		keys.put("f12",KeyEvent.VK_F12);
	}
	
	public void run(Command mainThread[], int mThreadLength, HashMap<String, Function> functions) {
		System.gc();
		
		threadIndex = 0;
		this.mainThread = mainThread;
		this.functions = functions;
		this.variables = new HashMap<String, String>();
		mainThreadLength = mThreadLength;
		jumpingIf = false;
		paused = false;
		incursionCount = 0;
		previousIndex = new int[64];
		
		//printScript();
		
		scriptThread = new Thread(this);
		scriptThread.start();
	}
	private void printScript() {
		System.out.println("# Commands: "+mainThreadLength);
		for (int i = 0; i < mainThreadLength; i++) {
			System.out.println(this.mainThread[i].id);
		}
		System.out.println("");
	}
	
	
	private void processCommand(Command cmd) {
		try {
			commands[cmd.id].execute(cmd.args);
		} catch (Exception e) {}
	}
	
	public void run() {
		int mouseX = MouseInfo.getPointerInfo().getLocation().x;
		while (threadIndex < mainThreadLength) {
			try {
				Thread.sleep(600);
			} catch (InterruptedException e) {}
			mouseX = MouseInfo.getPointerInfo().getLocation().x;
			
			while (!paused && threadIndex < mainThreadLength) {
				if (mouseX != MouseInfo.getPointerInfo().getLocation().x) {
					paused = true;
					Main.main.setPauseBtnText("Resume");
				}
				mouseX = MouseInfo.getPointerInfo().getLocation().x;
				if (jumpingIf) {
					if (mainThread[threadIndex].id == 7) {
						processCommand(mainThread[threadIndex]);
					}
				} else {
					processCommand(mainThread[threadIndex]);
				}
				//System.out.println("I-"+threadIndex);
				threadIndex++;
				mouseX = MouseInfo.getPointerInfo().getLocation().x;
				
				try {
					Thread.sleep(2);
				} catch (InterruptedException e) {}
			}
		}
		Main.main.goNormalUi();
	}
	void setVariable(String name, String data) {
		variables.put(name,data);
	}
	String getVariable(String name) {
		return variables.get(name);
	}
	Function getFunction(String name) {
		return functions.get(name);
	}
	void setThreadIndex(int newIndex) {
		if (incursionCount != 0) {
			if (threadIndex != previousIndex[incursionCount-1]) {
				previousIndex[incursionCount] = threadIndex;
				incursionCount++;
			}
		} else {
			previousIndex[0] = threadIndex;
			incursionCount = 1;
		}
		threadIndex = newIndex;
	}
	void setThreadIndexNoIncursion(int newIndex) {
		threadIndex = newIndex;
	}
	void returnThread() {
		if (incursionCount == 0) {
			threadIndex = mainThreadLength+1;
			return;
		}
		incursionCount--;
		threadIndex = previousIndex[incursionCount];
	}
	void stopScript() {
		threadIndex = mainThreadLength;
	}
}
